/*****************************************************************************
 *
 *  Copyright (C) 2003 Cdric Brgardis <cedric.bregardis@free.fr>
 *
 *  This file is part of BRIQUOLO
 *
 *  BRIQUOLO is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  BRIQUOLO is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with BRIQUOLO; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/
#include "Balle.h"
#include "TraceBalle.h"
#include <MOGL_Temps.h>
#include <MOGL_GestionnaireObjet.h>
#include <MOGL_MatriceTransformation.h>
#include "BibliothequeSon.h"
#include "Constante.h"
#ifndef M_PI
#define M_PI 3.14159265359
#endif

MOGL_Objet * Balle::_ObjetSource=NULL;

Balle::Balle(): ElementGraphique(), MOGL_Action(), _VitesseX(0), _VitesseY(0),
                _FacteurTaille(1), _RayonInit(0.5), _SuperPuissance(false), _TempoSuperPuissance(10, false, false),
                _CanalSonSuperPuissance(-1)
{
  if (_ObjetSource==NULL)
  {
    PreCharger(Texture::GetGestionnaireTexture());
  }
  _ElementArbre=new MOGL_Objet(*_ObjetSource);
  _TraceBalle=new TraceBalle(this);
  ((MOGL_Objet *)_ElementArbre)->AjouterElement(_TraceBalle);
  _Rayon=_RayonInit;
  _TempoSuperPuissance.FinTemporisation.Connecter(this, &Balle::_OnFinSuperPuissance);
  ((MOGL_Objet *)_ElementArbre)->AjouterElement(&_TempoSuperPuissance);
}

Balle::Balle(const Balle & p_Balle): ElementGraphique(p_Balle), _RayonInit(p_Balle._RayonInit), _CanalSonSuperPuissance(-1)
{
  _ElementArbre=new MOGL_Objet(*_ObjetSource);

  _TraceBalle=new TraceBalle(this);
  _SuperPuissance=p_Balle._SuperPuissance;
  _TraceBalle->SetSuperPuissance(_SuperPuissance);
  ((MOGL_Objet *)_ElementArbre)->AjouterElement(_TraceBalle);
  _VitesseX=p_Balle._VitesseX;
  _VitesseY=p_Balle._VitesseY;
  _ArriveeX=p_Balle._ArriveeX;
  _ArriveeY=p_Balle._ArriveeY;
  _Rayon=p_Balle._Rayon;
  _FacteurTaille=p_Balle._FacteurTaille;
  _TempoSuperPuissance=p_Balle._TempoSuperPuissance;
  _TempoSuperPuissance.FinTemporisation.Connecter(this, &Balle::_OnFinSuperPuissance);
  ((MOGL_Objet *)_ElementArbre)->AjouterElement(&_TempoSuperPuissance);
  _MatriceRotation=p_Balle._MatriceRotation;

  val=12;
}

Balle::~Balle()
{
  if (_CanalSonSuperPuissance!=-1)
  {
    Son::ArreterCanal(_CanalSonSuperPuissance);
  }
  delete _TraceBalle;
  delete _ElementArbre;
}

void Balle::PreCharger(MOGL_GestionnaireTexture * p_GestionnaireTexture)
{
  _ObjetSource=new MOGL_Objet;
  MOGL_GestionnaireObjet::ChargerObjetASCTriangle((Constante::GetGlobalDir()+Constante::GetDataDir()+"/balle.tri").c_str(), *p_GestionnaireTexture, *_ObjetSource);
}

void Balle::PreparerDeplacement(double p_Facteur)
{
  //SetPosition(_X+MOGL_Temps::GetVariationDistance(_VitesseX), _Y+MOGL_Temps::GetVariationDistance(_VitesseY));
  _ArriveeX=_X+MOGL_Temps::GetVariationDistance(_VitesseX)*p_Facteur;
  _ArriveeY=_Y+MOGL_Temps::GetVariationDistance(_VitesseY)*p_Facteur;

  if (_SuperPuissance && _CanalSonSuperPuissance==-1)
  {
    _CanalSonSuperPuissance=BibliothequeSon::SonSuperPuissance->Jouer(true);
  }
}

void Balle::SetVitesse(double p_VitesseX, double p_VitesseY)
{
  _VitesseX=p_VitesseX;
  _VitesseY=p_VitesseY;
}

double Balle::GetVitesseX() const
{
  return _VitesseX;
}

double Balle::GetVitesseY() const
{
  return _VitesseY;
}

double Balle::GetArriveeX() const
{
  return _ArriveeX;
}

double Balle::GetArriveeY() const
{
  return _ArriveeY;
}

double Balle::GetRayon() const
{
  return _Rayon;
}

void Balle::SetPosition(double p_X, double p_Y)
{
  _X=p_X;
  _Y=p_Y;
  if (_ElementArbre!=NULL)
  {
    double vitesse=sqrt(_VitesseX*_VitesseX + _VitesseY*_VitesseY);
    if (vitesse!=0)
    {
      double distance=MOGL_Temps::GetVariationTemps()*vitesse;
      double angleRot=distance/_Rayon;
    
      MOGL_Struct_Vecteur vec;
      vec.x=-_VitesseY/vitesse;
      vec.y=_VitesseX/vitesse;
      vec.z=0;

      MOGL_MatriceTransformation matRot=MOGL_MatriceTransformation::FabriqueRotation(vec, angleRot*180.0/M_PI);
      _MatriceRotation.Multiplier(matRot);
    }

    MOGL_MatriceTransformation mat=MOGL_MatriceTransformation::FabriqueEchelle(_FacteurTaille, _FacteurTaille, _FacteurTaille);
    mat.Multiplier(_MatriceRotation);
    

    _ElementArbre->ReinitialiserPosition();
    _ElementArbre->AjouteTransformation(mat);

    _ElementArbre->AjouterTranslation(_X,_Y,0);
  }
}

void Balle::SetTaille(double p_Facteur)
{
  _Rayon*=p_Facteur;
  _FacteurTaille*=p_Facteur;
  
  MOGL_MatriceTransformation mat=MOGL_MatriceTransformation::FabriqueEchelle(_FacteurTaille, _FacteurTaille, _FacteurTaille);
  mat.Multiplier(_MatriceRotation);

  _ElementArbre->ReinitialiserPosition();
  _ElementArbre->AjouteTransformation(mat);

  _ElementArbre->AjouterTranslation(_X,_Y,0);
}

bool Balle::GetSuperPuissance() const
{
  return _SuperPuissance;
}

void Balle::SetSuperPuissance(bool p_Actif)
{
  if (!_SuperPuissance && p_Actif && _CanalSonSuperPuissance==-1)
  {
    _CanalSonSuperPuissance=BibliothequeSon::SonSuperPuissance->Jouer(true);
  }

  if (_SuperPuissance && !p_Actif && _CanalSonSuperPuissance!=-1)
  {
    Son::ArreterCanal(_CanalSonSuperPuissance);
    _CanalSonSuperPuissance=-1;
  }

  _SuperPuissance=p_Actif;
  _TraceBalle->SetSuperPuissance(p_Actif);
  if (p_Actif)
  {
    _TempoSuperPuissance.Arreter();
    _TempoSuperPuissance.Demarrer();
  }
  else
  {
    _TempoSuperPuissance.Arreter();
  }
}

void Balle::_OnFinSuperPuissance()
{
  SetSuperPuissance(false);
}
