/*****************************************************************************
 *
 *  Copyright (C) 2003 Cdric Brgardis <cedric.bregardis@free.fr>
 *
 *  This file is part of BRIQUOLO
 *
 *  BRIQUOLO is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  BRIQUOLO is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with BRIQUOLO; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/
#include "BriqueBase.h"
#include "ExplosionBrique.h"

#include <MOGL_GestionnaireObjet.h>
#include <MOGL_Temps.h>
#include <iostream>
#include "BibliothequeSon.h"

#ifndef M_PI
#define M_PI 3.14159265359
#endif

BriqueBase::BriqueBase(TableauBase * p_TableauBase): ElementTableau(p_TableauBase), _TailleX(3), _TailleY(2), _Z(0), _VitesseZ(9)
{
  _Z=2*M_PI*(rand()/(RAND_MAX+0.0))-1;
}

BriqueBase::BriqueBase(const BriqueBase & p_BriqueBase): ElementTableau(p_BriqueBase),
                                                         _TailleX(p_BriqueBase._TailleX), _TailleY(p_BriqueBase._TailleY),
                                                         _VitesseZ(9)
{
  _Z=2*M_PI*(rand()/(RAND_MAX+0.0))-1;
}

BriqueBase::~BriqueBase()
{
}


void BriqueBase::Maj()
{
  _Z+=MOGL_Temps::GetVariationDistance(_VitesseZ);
  if (_Z>=2*M_PI)
  {
    _Z-=static_cast<float>(2*M_PI);
  }
  SetPosition(_X,_Y);
}

void BriqueBase::SetPosition(double p_X, double p_Y)
{
  _X=p_X;
  _Y=p_Y;
  if (_ElementArbre!=NULL)
  {
    _ElementArbre->ReinitialiserPosition();
    _ElementArbre->AjouterTranslation(_X,_Y,sin(_Z)/4);
  }
}

Struct_Collision BriqueBase::TesterCollision(Balle * p_Balle)
{
  Struct_Collision collision;
  collision.PositionCollision=1;
  collision.VitesseX=p_Balle->GetVitesseX();
  collision.VitesseY=p_Balle->GetVitesseY();

  if (_DerniereCollision==this && _DerniereBalle==p_Balle)
  {
    return collision;
  }

  double rayon=p_Balle->GetRayon();
  double x1=p_Balle->GetPositionX();
  double y1=p_Balle->GetPositionY();
  double x2=p_Balle->GetArriveeX();
  double y2=p_Balle->GetArriveeY();

  if ((x1+rayon<_X-_TailleX/2 && x2+rayon<_X-_TailleX/2) ||
      (x1-rayon>_X+_TailleX/2 && x2-rayon>_X+_TailleX/2) ||
      (y1+rayon<_Y-_TailleY/2 && y2+rayon<_Y-_TailleY/2) ||
      (y1-rayon>_Y+_TailleY/2 && y2-rayon>_Y+_TailleY/2))
  {
    return collision;
  }

  if (p_Balle->GetVitesseX()>0)
  {
    double tInter=(_X-_TailleX/2-rayon-p_Balle->GetPositionX())/(p_Balle->GetArriveeX()-p_Balle->GetPositionX());
    double yInter=tInter * (p_Balle->GetArriveeY()-p_Balle->GetPositionY()) + p_Balle->GetPositionY();

    if (tInter>0 && tInter<=1)
    {
      if (yInter>_Y-_TailleY/2 && yInter<_Y+_TailleY/2)
      {
        collision.VitesseX=-collision.VitesseX;
        collision.PositionCollision=tInter;
        collision.PositionX=_X-_TailleX/2-rayon;
        collision.PositionY=yInter;
      }
    }
  }
  else
  {
    double tInter=(_X+_TailleX/2+rayon-p_Balle->GetPositionX())/(p_Balle->GetArriveeX()-p_Balle->GetPositionX());
    double yInter=tInter * (p_Balle->GetArriveeY()-p_Balle->GetPositionY()) + p_Balle->GetPositionY();

    if (tInter>0 && tInter<=1)
    {
      if (yInter>_Y-_TailleY/2 && yInter<_Y+_TailleY/2)
      {
        collision.VitesseX=-collision.VitesseX;
        collision.PositionCollision=tInter;
        collision.PositionX=_X+_TailleX/2+rayon;
        collision.PositionY=yInter;
      }
    }
  }
  if (p_Balle->GetVitesseY()>0)
  {
    double tInter=(_Y-_TailleY/2-rayon-p_Balle->GetPositionY())/(p_Balle->GetArriveeY()-p_Balle->GetPositionY());
    double xInter=tInter * (p_Balle->GetArriveeX()-p_Balle->GetPositionX()) + p_Balle->GetPositionX();

    if (tInter>0 && tInter<=1)
    {
      if (xInter>_X-_TailleX/2 && xInter<_X+_TailleX/2)
      {
        collision.VitesseY=-collision.VitesseY;
        collision.PositionCollision=tInter;
        collision.PositionX=xInter;
        collision.PositionY=_Y-_TailleY/2-rayon;
      }
    }
  }
  else
  {
    double tInter=(_Y+_TailleY/2+rayon-p_Balle->GetPositionY())/(p_Balle->GetArriveeY()-p_Balle->GetPositionY());
    double xInter=tInter * (p_Balle->GetArriveeX()-p_Balle->GetPositionX()) + p_Balle->GetPositionX();

    if (tInter>0 && tInter<=1)
    {
      if (xInter>_X-_TailleX/2 && xInter<_X+_TailleX/2)
      {
        collision.VitesseY=-collision.VitesseY;
        collision.PositionCollision=tInter;
        collision.PositionX=xInter;
        collision.PositionY=_Y+_TailleY/2+rayon;
      }
    }
  }

  if (collision.PositionCollision==1)
  {
    double t1, t2, xRes1, yRes1, xRes2, yRes2;
    double xc,yc;

    // infrieur gauche
    xc=_X-_TailleX/2;
    yc=_Y-_TailleY/2;

    if (_IntersectionCoin(p_Balle, xc, yc, t1, xRes1, yRes1, t2, xRes2, yRes2))
    {
      if (xRes1<=xc && yRes1<=yc && t1<=1 && t1>0)
      {
        _SetCollisionCoin(p_Balle, xc, yc, t1, xRes1, yRes1,collision);
      }
      if(xRes2<=xc && yRes2<=yc && t2<=1 && t2>0 && t2<collision.PositionCollision)
      {
        _SetCollisionCoin(p_Balle, xc, yc, t2, xRes2, yRes2,collision);
      }
    }

    
    // infrieur droite
    xc=_X+_TailleX/2;
    yc=_Y-_TailleY/2;
    if (_IntersectionCoin(p_Balle, xc, yc, t1, xRes1, yRes1, t2, xRes2, yRes2))
    {
      if(xRes1>=xc && yRes1<=yc && t1<=1 && t1>0)
      {
        _SetCollisionCoin(p_Balle, xc, yc, t1, xRes1, yRes1,collision);
      }
      if(xRes2>=xc && yRes2<=yc && t2<=1 && t2>0 && t2<collision.PositionCollision)
      {
        _SetCollisionCoin(p_Balle, xc, yc, t2, xRes2, yRes2,collision);
      }
    }

    // suprieur droite
    xc=_X+_TailleX/2;
    yc=_Y+_TailleY/2;
    if (_IntersectionCoin(p_Balle, xc, yc, t1, xRes1, yRes1, t2, xRes2, yRes2))
    {
      if(xRes1>=xc && yRes1>=yc && t1<=1 && t1>0)
      {
        _SetCollisionCoin(p_Balle, xc, yc, t1, xRes1, yRes1,collision);
      }
      if(xRes2>=xc && yRes2>=yc && t2<=1 && t2>0 && t2<collision.PositionCollision)
      {
        _SetCollisionCoin(p_Balle, xc, yc, t2, xRes2, yRes2,collision);
      }
    }

    // suprieur gauche
    xc=_X-_TailleX/2;
    yc=_Y+_TailleY/2;
    if (_IntersectionCoin(p_Balle, xc, yc, t1, xRes1, yRes1, t2, xRes2, yRes2))
    {
      if(xRes1<=xc && yRes1>=yc && t1<=1 && t1>0)
      {
        _SetCollisionCoin(p_Balle, xc, yc, t1, xRes1, yRes1,collision);
      }
      if(xRes2<=xc && yRes2>=yc && t2<=1 && t2>0 && t2<collision.PositionCollision)
      {
        _SetCollisionCoin(p_Balle, xc, yc, t2, xRes2, yRes2,collision);
      }
    }
  }

  return collision;
}

void BriqueBase::_SetCollisionCoin(Balle * p_Balle, double p_CentreX, double p_CentreY,
                               double p_T, double p_ResX, double p_ResY,
                               Struct_Collision & p_Collision)
{
  p_Collision.PositionCollision=p_T;
  p_Collision.PositionX=p_ResX;
  p_Collision.PositionY=p_ResY;

  double vitesse=sqrt(p_Balle->GetVitesseX()*p_Balle->GetVitesseX() + p_Balle->GetVitesseY()*p_Balle->GetVitesseY());
  double beta=atan2(p_Balle->GetVitesseY() , p_Balle->GetVitesseX());
  double gama=atan2(p_ResY-p_CentreY , p_ResX-p_CentreX);

  p_Collision.VitesseX=-vitesse*cos(2*gama-beta);
  p_Collision.VitesseY=-vitesse*sin(2*gama-beta);
}


bool BriqueBase::_IntersectionCoin(Balle * p_Balle, double p_CentreX, double p_CentreY,
                               double & p_T1, double & p_ResX1, double & p_ResY1,
                               double & p_T2, double & p_ResX2, double & p_ResY2)
{
  double x1=p_Balle->GetPositionX();
  double x2=p_Balle->GetArriveeX();
  double y1=p_Balle->GetPositionY();
  double y2=p_Balle->GetArriveeY();

  double rayon=p_Balle->GetRayon();

  double a=y2*y2 - 2*y1*y2 + y1*y1 + x2*x2 - 2*x1*x2 + x1*x1;
  double b=-2*y2*p_CentreY + 2* y1*p_CentreY + 2*y1*y2 -2*y1*y1 -2*x2*p_CentreX + 2*x1*p_CentreX + 2*x1*x2 - 2*x1*x1;
  double c=p_CentreY*p_CentreY - 2*y1*p_CentreY + y1*y1 + p_CentreX*p_CentreX - 2*x1*p_CentreX +x1*x1 -rayon*rayon;

  double delta=b*b-4*a*c;
  if (delta>=0)
  {
    p_T1=(-b-sqrt(delta)) / (2*a);
    p_T2=(-b+sqrt(delta)) / (2*a);
    
    p_ResX1=(x2-x1)*p_T1+x1;
    p_ResX2=(x2-x1)*p_T2+x1;

    p_ResY1=(y2-y1)*p_T1+y1;
    p_ResY2=(y2-y1)*p_T2+y1;
   
    return true;
  }
  else
  {
    return false;
  }
}
