/*****************************************************************************
 *
 *  Copyright (C) 2003 Cdric Brgardis <cedric.bregardis@free.fr>
 *
 *  This file is part of BRIQUOLO
 *
 *  BRIQUOLO is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  BRIQUOLO is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with BRIQUOLO; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/
#include "MOGL_SaisieChaine.h"

MOGL_SaisieChaine::MOGL_SaisieChaine(MOGL_Panneau * p_Panneau, const string & p_Label, const string & p_Chaine)
  : MOGL_ElementPanneau(p_Panneau, true), _Label(p_Label), _Chaine(p_Chaine)
{
  _CouleurTexte.r=1;
  _CouleurTexte.g=1;
  _CouleurTexte.b=1;
  _CouleurTexte.a=1;
}

void MOGL_SaisieChaine::Afficher()
{
  MOGL_ElementPanneau::Afficher();

  string texte(_Label);
  texte+=" : ";
  texte+=_Chaine;

  unsigned int taille=_Police->GetTailleXChaine(texte);

  if (_Focus)
  {
    _Police->Afficher(texte, 0, 0, _CouleurTexte, _CouleurFondFocus);
    glDisable(GL_TEXTURE_2D);
    glBegin(GL_QUADS);
    glVertex2f(taille,0);
    glVertex2f(taille, _Police->GetTailleYCaractere('A'));
    glVertex2f(taille + _Police->GetTailleXCaractere('A'), _Police->GetTailleYCaractere('A'));
    glVertex2f(taille + _Police->GetTailleXCaractere('A'), 0);
    glEnd();
  }
  else
  {
    _Police->Afficher(texte, 0, 0, _CouleurTexte);
  }
}

void MOGL_SaisieChaine::SetLabel(const string & p_Label)
{
  _Label=p_Label;
}

string MOGL_SaisieChaine::GetLabel() const
{
  return _Label;
}

void MOGL_SaisieChaine::ClavierUp(SDL_keysym * key)
{
  if ( (key->sym>=SDLK_a && key->sym<=SDLK_z) ||
       (key->sym==SDLK_UNDERSCORE))
  {
    _Chaine+=key->sym;
    ChangementChaine.Emettre(_Chaine);
  }

  if (key->sym>=SDLK_KP0 && key->sym<=SDLK_KP9)
  {
    _Chaine+=key->sym - SDLK_KP0 + '0';
    ChangementChaine.Emettre(_Chaine);
  }

  switch (key->sym)
  {
    case SDLK_LEFT :
    case SDLK_BACKSPACE :
    {
      if (_Chaine.size()>0)
      {
        _Chaine.resize(_Chaine.size()-1);
        ChangementChaine.Emettre(_Chaine);
      }
      break;
    }
    default:
    {
      break;
    }
  }  
}

unsigned int MOGL_SaisieChaine::GetHauteurElement() const
{
  return _Police->GetTailleYMax();
}

unsigned int MOGL_SaisieChaine::GetLargeurElement() const
{
  string texte(_Label);
  texte+=" : ";
  texte+=_Chaine;

  return _Police->GetTailleXChaine(texte);
}

void MOGL_SaisieChaine::SetCouleurLabel(const MOGL_Struct_Couleur & p_Couleur)
{
  _CouleurTexte=p_Couleur;
}

MOGL_Struct_Couleur MOGL_SaisieChaine::GetCouleurLabel() const
{
  return _CouleurTexte;
}

string MOGL_SaisieChaine::GetChaine() const
{
  return _Chaine;
}

void MOGL_SaisieChaine::SetChaine(const string & p_Chaine)
{
  _Chaine=p_Chaine;
}
