/*****************************************************************************
 *
 *  Copyright (C) 2003 Cdric Brgardis <cedric.bregardis@free.fr>
 *
 *  This file is part of BRIQUOLO
 *
 *  BRIQUOLO is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  BRIQUOLO is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with BRIQUOLO; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/
#include "Plateau.h"
#include <MOGL_Temps.h>
#include <MOGL_GestionnaireObjet.h>
#include <MOGL_Noeud.h>
#include "BibliothequeSon.h"
#include "Constante.h"

#ifndef M_PI
#define M_PI 3.14159265359
#endif


MOGL_Objet * Plateau::_ObjetSourceMilieu=NULL;
MOGL_Objet * Plateau::_ObjetSourceDroite=NULL;
MOGL_Objet * Plateau::_ObjetSourceGauche=NULL;

Plateau::Plateau(): ElementGraphique(), _TailleInitX(8), _TailleInitY(2),
                    _TailleX(_TailleInitX), _TailleY(_TailleInitY),
                    _FacteurTaille(1)
{
  if (_ObjetSourceMilieu==NULL)
  {
    PreCharger();
  }
  _ElementArbre=new MOGL_Noeud;

  _ObjetMilieu=new MOGL_Objet(*_ObjetSourceMilieu);
  _ObjetDroite=new MOGL_Objet(*_ObjetSourceDroite);
  _ObjetDroite->ReinitialiserPosition();
  _ObjetDroite->AjouterTranslation(_TailleX/2,0,0);
  _ObjetGauche=new MOGL_Objet(*_ObjetSourceGauche);
  _ObjetGauche->ReinitialiserPosition();
  _ObjetGauche->AjouterTranslation(-_TailleX/2,0,0);

  ((MOGL_Noeud *)_ElementArbre)->AjouterElement(_ObjetMilieu);
  ((MOGL_Noeud *)_ElementArbre)->AjouterElement(_ObjetDroite);
  ((MOGL_Noeud *)_ElementArbre)->AjouterElement(_ObjetGauche);
}

Plateau::~Plateau()
{
  delete _ElementArbre;
  delete _ObjetMilieu;
  delete _ObjetGauche;
  delete _ObjetDroite;
}

Plateau::Plateau(const Plateau & p_Plateau): ElementGraphique(p_Plateau),
                                             _TailleInitX(p_Plateau._TailleInitX), _TailleInitY(p_Plateau._TailleInitY),
                                             _TailleX(p_Plateau._TailleX), _TailleY(p_Plateau._TailleY),
                                             _FacteurTaille(p_Plateau._FacteurTaille)
{
  if (_ObjetSourceMilieu==NULL)
  {
    PreCharger();
  }
  MOGL_Noeud * no=(MOGL_Noeud *)&p_Plateau._ElementArbre;
  _ElementArbre=new MOGL_Noeud(*no);

  _ObjetMilieu=new MOGL_Objet(*p_Plateau._ObjetMilieu);
  _ObjetDroite=new MOGL_Objet(*p_Plateau._ObjetDroite);
  _ObjetGauche=new MOGL_Objet(*p_Plateau._ObjetGauche);

  ((MOGL_Noeud *)_ElementArbre)->AjouterElement(_ObjetMilieu);
  ((MOGL_Noeud *)_ElementArbre)->AjouterElement(_ObjetDroite);
  ((MOGL_Noeud *)_ElementArbre)->AjouterElement(_ObjetGauche);

  _MinX=p_Plateau._MinX;
  _MaxX=p_Plateau._MaxX;
}

void Plateau::PreCharger()
{
  _ObjetSourceMilieu=new MOGL_Objet;
  MOGL_GestionnaireObjet::ChargerObjetASCTriangle((Constante::GetGlobalDir()+Constante::GetDataDir()+"/plateau_milieu.tri").c_str(), *Texture::GetGestionnaireTexture(), *_ObjetSourceMilieu);

  _ObjetSourceDroite=new MOGL_Objet;
  MOGL_GestionnaireObjet::ChargerObjetASCTriangle((Constante::GetGlobalDir()+Constante::GetDataDir()+"/plateau_droite.tri").c_str(), *Texture::GetGestionnaireTexture(), *_ObjetSourceDroite);

  _ObjetSourceGauche=new MOGL_Objet;
  MOGL_GestionnaireObjet::ChargerObjetASCTriangle((Constante::GetGlobalDir()+Constante::GetDataDir()+"/plateau_gauche.tri").c_str(), *Texture::GetGestionnaireTexture(), *_ObjetSourceGauche);
}

Struct_Collision Plateau::TesterCollision(Balle * p_Balle)
{
  Struct_Collision collision;
  collision.PositionCollision=1;
  collision.VitesseX=p_Balle->GetVitesseX();
  collision.VitesseY=p_Balle->GetVitesseY();

  if (p_Balle->GetVitesseY()<=0)
  {
    double rayon=p_Balle->GetRayon();
  
    
    double tInter=(_Y+_TailleY/2+rayon-p_Balle->GetPositionY())/(p_Balle->GetArriveeY()-p_Balle->GetPositionY());
    double xInter=tInter * (p_Balle->GetArriveeX()-p_Balle->GetPositionX()) + p_Balle->GetPositionX();

    if (tInter>0 && tInter<1)
    {
      if (xInter>_X-_TailleX/2 && xInter<_X+_TailleX/2)
      {
        collision.VitesseY=-collision.VitesseY;
        collision.PositionCollision=tInter;
        collision.PositionX=xInter;
        collision.PositionY=_Y+_TailleY/2+rayon;
      }
    }

    if (collision.PositionCollision==1)
    {
      double t1, t2, xRes1, yRes1, xRes2, yRes2;
      double xc,yc;

      // gauche
      xc=_X-_TailleX/2;
      yc=_Y;

      if (_IntersectionCoin(p_Balle, xc, yc, t1, xRes1, yRes1, t2, xRes2, yRes2))
      {
        if (xRes1<=xc && t1<1 && t1>0)
        {
          _SetCollisionCoin(p_Balle, xc, yc, t1, xRes1, yRes1,collision);
        }
        if(xRes2<=xc && t2<1 && t2>0 && t2<collision.PositionCollision)
        {
          _SetCollisionCoin(p_Balle, xc, yc, t2, xRes2, yRes2,collision);
        }
      }

      // droite
      xc=_X+_TailleX/2;
      yc=_Y;
      if (_IntersectionCoin(p_Balle, xc, yc, t1, xRes1, yRes1, t2, xRes2, yRes2))
      {
        if(xRes1>=xc && t1<1 && t1>0)
        {
          _SetCollisionCoin(p_Balle, xc, yc, t1, xRes1, yRes1,collision);
        }
        if(xRes2>=xc && t2<1 && t2>0 && t2<collision.PositionCollision)
        {
          _SetCollisionCoin(p_Balle, xc, yc, t2, xRes2, yRes2,collision);
        }
      }
    }
  }
  return collision;  
}

void Plateau::_SetCollisionCoin(Balle * p_Balle, double p_CentreX, double p_CentreY,
                               double p_T, double p_ResX, double p_ResY,
                               Struct_Collision & p_Collision)
{
  p_Collision.PositionCollision=p_T;
  p_Collision.PositionX=p_ResX;
  p_Collision.PositionY=p_ResY;

  double vitesse=sqrt(p_Balle->GetVitesseX()*p_Balle->GetVitesseX() + p_Balle->GetVitesseY()*p_Balle->GetVitesseY());
  double beta=atan2(-p_Balle->GetVitesseY() , -p_Balle->GetVitesseX());
  double gama=atan2(p_ResY-p_CentreY , p_ResX-p_CentreX);
  
  if (beta-gama>-M_PI/2 && beta-gama<M_PI/2)
  {
    p_Collision.VitesseX=vitesse*cos(2*gama-beta);
    p_Collision.VitesseY=fabs(vitesse*sin(2*gama-beta));
  }
  else
  {
    p_Collision.VitesseX=p_Balle->GetVitesseX();
    p_Collision.VitesseY=p_Balle->GetVitesseY();
  }     
}


bool Plateau::_IntersectionCoin(Balle * p_Balle, double p_CentreX, double p_CentreY,
                               double & p_T1, double & p_ResX1, double & p_ResY1,
                               double & p_T2, double & p_ResX2, double & p_ResY2)
{
  double x1=p_Balle->GetPositionX();
  double x2=p_Balle->GetArriveeX();
  double y1=p_Balle->GetPositionY();
  double y2=p_Balle->GetArriveeY();

  double rayon=p_Balle->GetRayon()+_TailleY/2;

  double a=y2*y2 - 2*y1*y2 + y1*y1 + x2*x2 - 2*x1*x2 + x1*x1;
  double b=-2*y2*p_CentreY + 2* y1*p_CentreY + 2*y1*y2 -2*y1*y1 -2*x2*p_CentreX + 2*x1*p_CentreX + 2*x1*x2 - 2*x1*x1;
  double c=p_CentreY*p_CentreY - 2*y1*p_CentreY + y1*y1 + p_CentreX*p_CentreX - 2*x1*p_CentreX +x1*x1 -rayon*rayon;

  double delta=b*b-4*a*c;
  if (delta>=0)
  {
    p_T1=(-b-sqrt(delta)) / (2*a);
    p_T2=(-b+sqrt(delta)) / (2*a);
    
    p_ResX1=(x2-x1)*p_T1+x1;
    p_ResX2=(x2-x1)*p_T2+x1;

    p_ResY1=(y2-y1)*p_T1+y1;
    p_ResY2=(y2-y1)*p_T2+y1;
   
    return true;
  }
  else
  {
    return false;
  }
}

void Plateau::CollisionDeplacement(double p_Distance, Balle * p_Balle)
{
  if (p_Balle->GetVitesseY()<=0)
  {
  
    double rayon=_TailleY/2+p_Balle->GetRayon();

    double alpha=asin((p_Balle->GetPositionY() - _Y)/(rayon));
    double x=rayon*cos(alpha);

    if ((p_Balle->GetPositionX()-x)>=(_X+_TailleX/2) && (p_Balle->GetPositionX()-x)<=(_X+_TailleX/2+p_Distance))
    {
      double vitesse=sqrt(p_Balle->GetVitesseX()*p_Balle->GetVitesseX() + p_Balle->GetVitesseY()*p_Balle->GetVitesseY());
      double beta=atan2(-p_Balle->GetVitesseY() , -p_Balle->GetVitesseX());
      
      if (beta-alpha>-M_PI/2 && beta-alpha<M_PI/2)
      {
        p_Balle->SetVitesse(vitesse*cos(2*alpha-beta),fabs( vitesse*sin(2*alpha-beta)));
        BibliothequeSon::SonRebond->Jouer(false);
      }
      else
      {
        p_Balle->SetVitesse(vitesse*cos(alpha),fabs( vitesse*sin(alpha)));
        BibliothequeSon::SonRebond->Jouer(false);
      }
    }
    
    rayon=_TailleY/2+p_Balle->GetRayon();

    alpha=asin((p_Balle->GetPositionY() - _Y)/(rayon));
    x=rayon*cos(alpha);

    if ((p_Balle->GetPositionX()+x)<=(_X-_TailleX/2) && (p_Balle->GetPositionX()+x)>=(_X-_TailleX/2+p_Distance))
    {
      double vitesse=sqrt(p_Balle->GetVitesseX()*p_Balle->GetVitesseX() + p_Balle->GetVitesseY()*p_Balle->GetVitesseY());
      double beta=atan2(-p_Balle->GetVitesseY() , p_Balle->GetVitesseX());
      
      if (beta-alpha>-M_PI/2 && beta-alpha<M_PI/2)
      {
        p_Balle->SetVitesse(-vitesse*cos(2*alpha-beta), fabs(vitesse*sin(2*alpha-beta)));
        BibliothequeSon::SonRebond->Jouer(false);
      }
      else
      {
        p_Balle->SetVitesse(-vitesse*cos(alpha), fabs(vitesse*sin(alpha)));
        BibliothequeSon::SonRebond->Jouer(false);
      }
    }
  }
}

void Plateau::Deplacer(double p_Distance)
{
  if (_X+p_Distance+_TailleX/2+_TailleY/2 > _MaxX)
  {
    SetPosition(_MaxX-_TailleX/2-_TailleY/2,_Y);
  }
  else if (_X+p_Distance-_TailleX/2-_TailleY/2 < _MinX)
  {
    SetPosition(_MinX+_TailleX/2+_TailleY/2,_Y);
  }
  else
  {
    SetPosition(_X+p_Distance,_Y);
  }
}

void Plateau::SetMinX(double p_Valeur)
{
  _MinX=p_Valeur;
}

void Plateau::SetMaxX(double p_Valeur)
{
  _MaxX=p_Valeur;
}

double Plateau::GetTailleX() const
{
  return _TailleX;
}

double Plateau::GetTailleY() const
{
  return _TailleY;
}

void Plateau::SetTaille(double p_TailleX)
{
  double facteur=p_TailleX/_TailleX;
  SetTailleFacteur(facteur);
}

void Plateau::SetTailleFacteur(double p_Facteur)
{
  _FacteurTaille*=p_Facteur;
  _TailleX*=p_Facteur;
  
  if (_X+_TailleX/2+_TailleY/2 > _MaxX)
  {
    _X=_MaxX-_TailleX/2-_TailleY/2;
  }
  else if (_X-_TailleX/2-_TailleY/2 < _MinX)
  {
    _X=_MinX+_TailleX/2+_TailleY/2;
  }

  MOGL_MatriceTransformation mat=MOGL_MatriceTransformation::FabriqueEchelle(_FacteurTaille, 1, 1);

  _ObjetMilieu->ReinitialiserPosition();
  _ObjetMilieu->AjouteTransformation(mat);

  _ObjetDroite->ReinitialiserPosition();
  _ObjetDroite->AjouterTranslation(_TailleX/2,0,0);

  _ObjetGauche->ReinitialiserPosition();
  _ObjetGauche->AjouterTranslation(-_TailleX/2,0,0);
  SetPosition(_X,_Y);
}
